/*
 * original:lejos-osek/src/nxtvm/platform/nxt/nxt_avr.c rev.1.1.1.1
 * modified history.
 * 05/24/2010: Modifier Takuya Ishikawa
 *           : Changed interface name to entry function name with TECS naming rule
 *           : Modified initialize to call link_init
 *           : Modified updateBody to call link_init only when twi is not ok
 *                                 to calculate count by exor.
 *           : Deleted nxt_avr_status
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 12/13/2009: Modifider Ryosuke Takeuchi
 *           : Modified nxt_avr_init for ITRON ATT_INI static API.
 *           : Modified nxt_avr_1kHz_update to callback function.
 *           : Modified nxt_avr_input_power function. 2nd argument bit2
 *             into io_to_avr.input_power bit4.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tAVRBody_template.c => src/tAVRBody.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]# AHA#[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * brainwashString  char_t*          ATTR_brainwashString
 * initializedFlag  bool_t           VAR_initializedFlag
 * ioToAvr          TOAVR            VAR_ioToAvr     
 * ioFromAvr        FROMAVR          VAR_ioFromAvr   
 * dataFromAvr      uint8_t*         VAR_dataFromAvr 
 * dataToAvr        uint8_t*         VAR_dataToAvr   
 * updateCount      uint8_t          VAR_updateCount 
 * linkInitWait     uint8_t          VAR_linkInitWait
 * linkRunning      bool_t           VAR_linkRunning 
 * linkState        uint32_t         VAR_linkState   
 *
 * Ăь֐ #_TCPF_#
 * call port : cTWI  signature: sTWI context: task
 *   bool_t         cTWI_initialize( );
 *   void           cTWI_startWrite( uint32_t dev_addr, uint32_t int_addr_bytes, uint32_t int_addr, const uint8_t* data, uint32_t nBytes );
 *   void           cTWI_startRead( uint32_t dev_addr, uint32_t int_addr_bytes, uint32_t int_addr, uint8_t* data, uint32_t nBytes );
 *   bool_t         cTWI_isBusy( );
 *   bool_t         cTWI_isOk( );
 *   void           cTWI_reset( );
 *
 * #[</PREAMBLE>]# */

#include "tAVRBody_tecsgen.h"

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

static void
nxt_avr_start_read(void)
{
	memset(VAR_dataFromAvr, 0, 13);
	cTWI_startRead(NXT_AVR_ADDRESS, 0, 0, VAR_dataFromAvr, 13);
}

static void
nxt_avr_start_send(void)
{
	int32_t i;
	uint8_t checkByte = 0;
	uint8_t *a = VAR_dataToAvr;
	uint8_t *b = (uint8_t *)(&VAR_ioToAvr);
	
	i = sizeof(VAR_ioToAvr);
	while(i){
		*a = *b;
		checkByte += *b;
		a++;
		b++;
		i--;
	}
	
	*a = ~checkByte;

	cTWI_startWrite(NXT_AVR_ADDRESS, 0, 0, VAR_dataToAvr, 9);
}

static void
nxt_avr_link_init(void)
{
	cTWI_startWrite(NXT_AVR_ADDRESS, 0, 0, (const uint8_t *)ATTR_brainwashString,
					strlen(ATTR_brainwashString));
}

static uint16_t
Unpack16(const uint8_t *x)
{
	uint16_t retval;

	retval = (((uint16_t) (x[0])) & 0x00ff) | ((((uint16_t) (x[1])) << 8) & 0xff00);
	return retval;
}

static void
nxt_avr_unpack(void)
{
	uint8_t check_sum;
	uint8_t *p;
	uint16_t buttonsVal;
	uint32_t voltageVal;
	int32_t i;

	p = VAR_dataFromAvr;

	for(check_sum = i = 0; i < 13; i++){
		check_sum += *p;
		p++;
	}

	if(check_sum != 0xff){
		return;
	}

	p = VAR_dataFromAvr;

	// Marshall
	for(i = 0; i < NXT_AVR_N_INPUTS; i++){
		VAR_ioFromAvr.adc_value[i] = Unpack16(p);
		p += 2;
	}

	buttonsVal = Unpack16(p);
	p += 2;

	VAR_ioFromAvr.buttons = 0;

	if(buttonsVal > 1023){
		VAR_ioFromAvr.buttons |= 1;
		buttonsVal -= 0x7ff;
	}

	if(buttonsVal > 720){
		VAR_ioFromAvr.buttons |= 0x08;
	} else if(buttonsVal > 270){
		VAR_ioFromAvr.buttons |= 0x04;
	} else if(buttonsVal > 60){
		VAR_ioFromAvr.buttons |= 0x02;
	}

	voltageVal = Unpack16(p);

	VAR_ioFromAvr.battery_is_AA = (voltageVal & 0x8000) ? 1 : 0;
	VAR_ioFromAvr.avr_fw_version_major = (voltageVal >> 13) & 3;
	VAR_ioFromAvr.avr_fw_version_minor = (voltageVal >> 10) & 7;


	// Figure out voltage
	// The units are 13.848 mV per bit.
	// To prevent fp, we substitute 13.848 with 14180/1024

	voltageVal &= 0x3ff;		// Toss unwanted bits.
	voltageVal *= 14180;
	voltageVal >>= 10;
	VAR_ioFromAvr.battery_mV = voltageVal;

}

static inline void
nxt_avr_updateBody(void)
{
	if(!VAR_initializedFlag){
		return;
	}

	if(VAR_linkInitWait){
		VAR_linkInitWait--;
		return;
	}

	if(!cTWI_isOk()){
		cTWI_initialize();
		memset(VAR_dataFromAvr, 0, 13);
		nxt_avr_link_init();
		VAR_linkInitWait = 2;
		VAR_updateCount = 0;
		return;
	}
	
	if(cTWI_isBusy()){
		return;
	}

	if(VAR_updateCount & 1){
		nxt_avr_start_read();
	} else {
		nxt_avr_unpack();
		nxt_avr_start_send();
	}
	
	VAR_updateCount ^= 0x01;
}

/* :֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eAVR
 * entry port: eAVR
 * signature:  sAVR
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eAVR_update
 * name:         eAVR_update
 * global_name:  tAVRBody_eAVR_update
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eAVR_update()
{
	nxt_avr_updateBody();
}

/* #[<ENTRY_FUNC>]# eAVR_setMotor
 * name:         eAVR_setMotor
 * global_name:  tAVRBody_eAVR_setMotor
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eAVR_setMotor(uint32_t portNumber, int32_t powerPercent, int32_t brake)
{
	if(portNumber < 3){
		VAR_ioToAvr.output_percent[portNumber] = powerPercent;

		if(brake){
			VAR_ioToAvr.output_mode |= (1 << portNumber);
		} else {
			VAR_ioToAvr.output_mode &= ~(1 << portNumber);
		}
	}
}

/* #[<ENTRY_FUNC>]# eAVR_powerDown
 * name:         eAVR_powerDown
 * global_name:  tAVRBody_eAVR_powerDown
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eAVR_powerDown()
{
	VAR_ioToAvr.power = 0x5a;
	VAR_ioToAvr.pwm_frequency = 0x00;
}

/* #[<ENTRY_FUNC>]# eAVR_getButton
 * name:         eAVR_getButton
 * global_name:  tAVRBody_eAVR_getButton
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
uint32_t
eAVR_getButton()
{
	return VAR_ioFromAvr.buttons;
}

/* #[<ENTRY_FUNC>]# eAVR_batteryVoltage
 * name:         eAVR_batteryVoltage
 * global_name:  tAVRBody_eAVR_batteryVoltage
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
uint32_t
eAVR_batteryVoltage()
{
	return VAR_ioFromAvr.battery_mV;
}

/* #[<ENTRY_FUNC>]# eAVR_getSensor
 * name:         eAVR_getSensor
 * global_name:  tAVRBody_eAVR_getSensor
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
uint32_t
eAVR_getSensor(uint32_t portNumber)
{
	if(portNumber < 4){
		return VAR_ioFromAvr.adc_value[portNumber];
	} else {
		return 0;
	}
}

/* #[<ENTRY_FUNC>]# eAVR_setInputPower
 * name:         eAVR_setInputPower
 * global_name:  tAVRBody_eAVR_setInputPower
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eAVR_setInputPower(uint32_t portNumber, uint32_t powerType)
{
	if(portNumber < 4 && powerType <= 2){
		VAR_ioToAvr.input_power &= ~(0x11 << portNumber);
		VAR_ioToAvr.input_power |= ((powerType & 0x2 ? 0x10 << portNumber : 0) | ((powerType & 1) << portNumber));
	}
}

/* #[<ENTRY_PORT>]# eInitialize
 * entry port: eInitialize
 * signature:  sInitializeRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eInitialize_main
 * name:         eInitialize_main
 * global_name:  tAVRBody_eInitialize_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eInitialize_main()
{
	cTWI_initialize();

	memset(&VAR_ioToAvr, 0, sizeof(VAR_ioToAvr));
	nxt_avr_link_init(); // ishikawa-add
	VAR_linkInitWait = 2;
	VAR_ioToAvr.power = 0;
	VAR_ioToAvr.pwm_frequency = 8;
	
	VAR_initializedFlag = true;
}

/* #[<ENTRY_PORT>]# eiCyclicBody
 * entry port: eiCyclicBody
 * signature:  siHandlerBody
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiCyclicBody_main
 * name:         eiCyclicBody_main
 * global_name:  tAVRBody_eiCyclicBody_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eiCyclicBody_main()
{
	nxt_avr_updateBody();
}

/* #[<POSTAMBLE>]#
 *   AEA導g[CWSCC
 * #[</POSTAMBLE>]#*/
