/*
 * original:lejos-osek/src/nxtvm/platform/nxt/display.c rev.1.1.1.1
 * modified history.
 * 05/24/2010: Modifier Takuya Ishikawa
 *           : Changed interface name to entry function name with TECS nameing rule
 *           : Modified initialize to
 *               display_clear(0); -> memset(...);;
 *               lcd_init(...); -> initialize(...); powerOn(...);
 *           : Added interface to powerOff(clear and powerOff lcd)
 *           : Changed display_unsigned_worker to display_convert
 *           : Modified putChar to plus currentX
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 04/03/2010: Modifider Ryosuke Takeuchi
 *           : Modified Device Interface to ITRON-SIL interface and Changed
 *             AT91SAM7.h to SIL defined include file.
 *           : Modified systick_wait_ms function to ITRON dly_tsk servec call function. 
 *           : Modified display_init function argument to ITRON ATT static API type.
 *           : Added spi refresh function.(display_refresh).
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tDisplayBody_template.c => src/tDisplayBody.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]# AHA#[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * width            int32_t          ATTR_width      
 * depth            int32_t          ATTR_depth      
 * cellWidth        int32_t          ATTR_cellWidth  
 * charWidth        int32_t          ATTR_charWidth  
 * charDepth        int32_t          ATTR_charDepth  
 * font             const uint8_t*   ATTR_font       
 * buffer           uint8_t*         VAR_buffer      
 * currentX         int32_t          VAR_currentX    
 * currentY         int32_t          VAR_currentY    
 *
 * Ăь֐ #_TCPF_#
 * call port : cLCD  signature: sLCD context: task
 *   void           cLCD_initialize( const uint8_t* display );
 *   void           cLCD_powerOn( const uint8_t* display );
 *   void           cLCD_powerOff( );
 *   void           cLCD_sendData( const uint8_t* buffer );
 *
 * #[</PREAMBLE>]# */

#include "tDisplayBody_tecsgen.h"
#include <sil.h>

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

static inline void
display_putchar(uint8_t c)
{
	uint8_t *tmpBuffer;
	int32_t i;

	if(c >= 0 && c < 128 &&
	   VAR_currentX >= 0 && VAR_currentX < ATTR_charWidth &&
	   VAR_currentY >= 0 && VAR_currentY < ATTR_charDepth){
		tmpBuffer = &VAR_buffer[VAR_currentY * ATTR_width + VAR_currentX * ATTR_cellWidth];

		for(i = 0; i < (ATTR_cellWidth - 1); i++){
			*tmpBuffer = ATTR_font[c * 5 + i];
			tmpBuffer++;
		}
		
		VAR_currentX++;
	}
}

static const char_t raddec[] = "0123456789";
static const char_t radHEX[] = "0123456789ABCDEF";

static inline void
display_convert(uintptr_t val, uint_t radix, const char_t *radchar, 
                int32_t places, bool_t minus)
{
	uint8_t digitBuffer[ATTR_charWidth - 2];
	int8_t i = 0;

	/*
	 *  𕶎ɕϊ
	 */
	do{
		if(i > (ATTR_charWidth - 2)){
			break;
		}
		
		digitBuffer[i++] = radchar[val % radix];
		val /= radix;
	} while(val);

	/*
	 *  Ȃ𖄂߂
	 */
	while((i + minus) < places){
		if(radix == 10){ // 10i
			display_putchar(' ');
		} else { // 16i
			display_putchar('0');
		}
		places--;
	}
	
	/*
	 *  ϊfBXvCɕ\
	 */
	if(minus){
		display_putchar('-');
	}
	while(i > 0){
		display_putchar(digitBuffer[--i]);
	}
}

/* 󂯌֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eDisplay
 * entry port: eDisplay
 * signature:  sDisplay
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eDisplay_update
 * name:         eDisplay_update
 * global_name:  tDisplayBody_eDisplay_update
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_update()
{
	cLCD_sendData((uint8_t *) VAR_buffer);
}

/* #[<ENTRY_FUNC>]# eDisplay_clear
 * name:         eDisplay_clear
 * global_name:  tDisplayBody_eDisplay_clear
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_clear(bool_t updateToo)
{
	memset(VAR_buffer, 0, ATTR_depth*ATTR_width);
	if(updateToo){
		cLCD_sendData((uint8_t *) VAR_buffer);
	}
}

/* #[<ENTRY_FUNC>]# eDisplay_gotoXY
 * name:         eDisplay_gotoXY
 * global_name:  tDisplayBody_eDisplay_gotoXY
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_gotoXY(int_t x, int_t y)
{
	VAR_currentX = x;
	VAR_currentY = y;
}

/* #[<ENTRY_FUNC>]# eDisplay_putChar
 * name:         eDisplay_putChar
 * global_name:  tDisplayBody_eDisplay_putChar
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_putChar(char_t c)
{
	display_putchar(c);
}

/* #[<ENTRY_FUNC>]# eDisplay_putString
 * name:         eDisplay_putString
 * global_name:  tDisplayBody_eDisplay_putString
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_putString(const char_t* string)
{
	while(*string){
		if(*string != '\n'){ /* s */
			display_putchar(*string);
		} else{
			VAR_currentX = 0;
			VAR_currentY++;
		}
		
		string++;
	}
}

/* #[<ENTRY_FUNC>]# eDisplay_putInt
 * name:         eDisplay_putInt
 * global_name:  tDisplayBody_eDisplay_putInt
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_putInt(int32_t val, uint32_t places)
{
	/*
	 *  ̗L
	 */
	if(val >= 0){
		display_convert(val, 10, raddec, places, false);
	} else {
		display_convert((-1 * val), 10, raddec, places, true);
	}
}

/* #[<ENTRY_FUNC>]# eDisplay_putUnsigned
 * name:         eDisplay_putUnsigned
 * global_name:  tDisplayBody_eDisplay_putUnsigned
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_putUnsigned(uint32_t val, uint32_t places)
{
	display_convert(val, 10, raddec, places, false);
}

/* #[<ENTRY_FUNC>]# eDisplay_putHex
 * name:         eDisplay_putHex
 * global_name:  tDisplayBody_eDisplay_putHex
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_putHex(uint32_t val, uint32_t places)
{
	display_convert(val, 16, radHEX, places, false);
}

/* #[<ENTRY_FUNC>]# eDisplay_getBuffer
 * name:         eDisplay_getBuffer
 * global_name:  tDisplayBody_eDisplay_getBuffer
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
uint8_t*
eDisplay_getBuffer()
{
	return VAR_buffer;
}

/* #[<ENTRY_FUNC>]# eDisplay_powerOff
 * name:         eDisplay_powerOff
 * global_name:  tDisplayBody_eDisplay_powerOff
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_powerOff(bool_t updateToo)
{
	memset(VAR_buffer, 0, ATTR_depth*ATTR_width);
	cLCD_sendData(VAR_buffer);
	
	sil_dly_nse(10*1000*1000);
	cLCD_powerOff();
	sil_dly_nse(10*1000*1000);
}

/* #[<ENTRY_FUNC>]# eDisplay_copyBitmap
 * name:         eDisplay_copyBitmap
 * global_name:  tDisplayBody_eDisplay_copyBitmap
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eDisplay_copyBitmap(const uint8_t* data, uint32_t width, uint32_t depth, uint32_t x, uint32_t y)
{
	uint32_t i, j;
	uint32_t dy, dx;

	for(i = 0; i < depth; i++){
		for(j = 0; j < width; j++){
			dx = x + j;
			dy = y + i;

			if(dx < ATTR_width && dy < ATTR_depth){
				VAR_buffer[(y + i) * ATTR_width + (x + j)] = data[width * i + j];
			}
		}
	}
}

/* #[<ENTRY_PORT>]# eInitialize
 * entry port: eInitialize
 * signature:  sInitializeRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eInitialize_main
 * name:         eInitialize_main
 * global_name:  tDisplayBody_eInitialize_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eInitialize_main()
{
	sil_dly_nse(1000*1000*1000);
	/*
	 *  obt@NA
	 */
	memset(VAR_buffer, 0, ATTR_depth*ATTR_width);

	/*
	 *  LCD
	 */
	cLCD_initialize((uint8_t *)VAR_buffer);
	cLCD_powerOn((uint8_t *)VAR_buffer);
}

/* #[<POSTAMBLE>]#
 *   AEA導g[CWSCC
 * #[</POSTAMBLE>]#*/
