/*
 * original:lejos-osek/src/nxtvm/platform/nxt/nxt_lcd.c rev.1.1.1.1
 * modified history.
 * 05/24/2010: Modifier Takuya Ishikawa
 *           : Changed interface name to entry function name with TECS naming rule
 *           : divided initialize to spi_initialize and powerOn
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 04/03/2010: Modifider Ryosuke Takeuchi
 *           : Modified systick_wait_ms function to TOPPERS/JSP soft delay timer.
 *           : Added nxt_lcd_power_down function for lcd reset.
 *           : Modified nxt_lcd_init and nxt_lcd_power_up argument
 *             for lcd buffer information.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tLCD_template.c => src/tLCD.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]# AHA#[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * width            int32_t          ATTR_width      
 * depth            int32_t          ATTR_depth      
 * display          uint8_t*         VAR_display     
 *
 * Ăь֐ #_TCPF_#
 * call port : cSPI  signature: sSPI context: task
 *   void           cSPI_initialize( );
 *   void           cSPI_write( uint32_t CD, const uint8_t* data, uint32_t nBytes );
 *   void           cSPI_setDisplay( const uint8_t* display );
 *   void           cSPI_refresh( );
 *
 * #[</PREAMBLE>]# */

#include "tLCD_tecsgen.h"
#include <sil.h>

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

static inline void
lcd_command(uint8_t cmd)
{
	uint8_t tmp = cmd;

	cSPI_write(0, &tmp, 1);
}

static inline void
lcd_set_col(uint32_t coladdr)
{
	lcd_command(0x00 | (coladdr & 0xF));
	lcd_command(0x10 | ((coladdr >> 4) & 0xF));
}

static inline void
lcd_set_multiplex_rate(uint32_t mr)
{
	lcd_command(0x20 | (mr & 3));
}

static inline void
lcd_set_temp_comp(uint32_t tc)
{
	lcd_command(0x24 | (tc & 3));
}

static inline void
lcd_set_panel_loading(uint32_t hi)
{
	lcd_command(0x28 | ((hi) ? 1 : 0));
}

static inline void
lcd_set_pump_control(uint32_t pc)
{
	lcd_command(0x2c | (pc & 3));
}

static inline void
lcd_set_scroll_line(uint32_t sl)
{
	lcd_command(0x40 | (sl & 0x3f));
}

static inline void
lcd_set_page_address(uint32_t pa)
{
	lcd_command(0xB0 | (pa & 0xf));
}

static inline void
lcd_set_pot(uint32_t pot)
{
	lcd_command(0x81);
	lcd_command(pot & 0xff);
}

static inline void
lcd_set_ram_address_control(uint32_t ac)
{
	lcd_command(0x88 | (ac & 7));
}

static inline void
lcd_set_frame_rate(uint32_t fr)
{
	lcd_command(0xA0 | (fr & 1));
}

static inline void
lcd_set_all_pixels_on(uint32_t on)
{
	lcd_command(0xA4 | ((on) ? 1 : 0));
}

static inline void
lcd_inverse_display(uint32_t on)
{
	lcd_command(0xA6 | ((on) ? 1 : 0));
}

static inline void
lcd_enable(uint32_t on)
{
	lcd_command(0xAE | ((on) ? 1 : 0));
}

static inline void
lcd_set_map_control(uint32_t map_control)
{
	lcd_command(0xC0 | ((map_control & 3) << 1));
}

static inline void
lcd_reset()
{
	lcd_command(0xE2);
}

static inline void
lcd_set_bias_ratio(uint32_t ratio)
{
	lcd_command(0xE8 | (ratio & 3));
}

static inline void
lcd_set_cursor_update(uint32_t on)
{
	lcd_command(0xEE | ((on) ? 1 : 0));
}


/* :֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eLCD
 * entry port: eLCD
 * signature:  sLCD
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eLCD_initialize
 * name:         eLCD_initialize
 * global_name:  tLCD_eLCD_initialize
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eLCD_initialize(const uint8_t* display)
{
	cSPI_initialize();
}

/* #[<ENTRY_FUNC>]# eLCD_powerOn
 * name:         eLCD_powerOn
 * global_name:  tLCD_eLCD_powerOn
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eLCD_powerOn(const uint8_t *display)
{
	sil_dly_nse(20*1000*1000);
	lcd_reset();
	sil_dly_nse(20*1000*1000);
	lcd_set_multiplex_rate(3);	// 1/65
	lcd_set_bias_ratio(3);	// 1/9
	lcd_set_pot(0x60);	// ?? 9V??

	lcd_set_ram_address_control(1); // auto wrap
	lcd_set_map_control(0x02); // mirror in y

	cSPI_setDisplay(display);
	lcd_enable(1);
}

/* #[<ENTRY_FUNC>]# eLCD_powerOff
 * name:         eLCD_powerOff
 * global_name:  tLCD_eLCD_powerOff
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eLCD_powerOff()
{
	lcd_reset();
}

/* #[<ENTRY_FUNC>]# eLCD_sendData
 * name:         eLCD_sendData
 * global_name:  tLCD_eLCD_sendData
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eLCD_sendData(const uint8_t* buffer)
{
	int32_t i;

	for(i = 0; i < ATTR_depth; i++){
		lcd_set_col(0);
		lcd_set_page_address(i);

		cSPI_write(1, buffer, ATTR_width);
		buffer += ATTR_width;
	}
}

/* #[<POSTAMBLE>]#
 *   AEA導g[CWSCC
 * #[</POSTAMBLE>]#*/
