/*
 * original:lejos-osek/src/nxtvm/platform/nxt/nxt_motor.c rev.1.1.1.1
 * modified history.
 * 05/25/2010: Modifier Takuya Ishikawa
 *           : Devided file nxt_motor.c -> tRotaryEncoderBody.c and tMotor.c
 *             This file has initialize, ISR, cyclic handler
 *           : Added terminate routine which stops motor
 *           : Changed interface name to entry function name with TECS naming rule
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 12/13/2009: Modifider Ryosuke Takeuchi
 *           : Modified Device Interface to ITRON-SIL interface and Changed
 *             AT91SAM7.h to SIL defined include file.
 *           : Modified nxt_motor_init for ITRON ATT_INI static API.
 *           : Modified nxt_motor_1kHz_process to callback function.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tMotorDriverBody_template.c => src/tMotorDriverBody.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]# AHA#[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * PIOBase          void*            ATTR_PIOBase    
 * interruptNumber  INTNO            ATTR_interruptNumber
 * interruptCount   int32_t          VAR_interruptCount
 * initializedFlag  bool_t           VAR_initializedFlag
 *
 * Ăь֐ #_TCPF_#
 * call port : ciMotorInterrupt  signature: siMotorInterrupt context: non-task
 *   void           ciMotorInterrupt_quadDecode( subscript, uint32_t pins );
 *   void           ciMotorInterrupt_stopMotor( subscript );
 *
 * #[</PREAMBLE>]# */

#include "tRotaryEncoderBody_tecsgen.h"
#include <sil.h>
#include "arm_gcc/nxt/at91sam7s.h"


#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

/* :֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eInitialize
 * entry port: eInitialize
 * signature:  sInitializeRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eInitialize_main
 * name:         eInitialize_main
 * global_name:  tRotaryEncoderBody_eInitialize_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eInitialize_main()
{
	sil_wrw_mem((void *)(TADR_PMC_BASE+TOFF_PMC_PCER), (1 << ATTR_interruptNumber));
	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_IDR), ~0);
	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_IFER), MOTOR_PIN_MASK);
	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_PUDR), MOTOR_PIN_MASK);
	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_PER),  MOTOR_PIN_MASK);
	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_ODR),  MOTOR_PIN_MASK);

	sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_IER),  MOTOR_INTERRUPT_PINS);
	VAR_initializedFlag = true;
}

/* #[<ENTRY_PORT>]# eTerminate
 * entry port: eTerminate
 * signature:  sTerminateRoutineBody
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eTerminate_main
 * name:         eTerminate_main
 * global_name:  tRotaryEncoderBody_eTerminate_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eTerminate_main()
{
	int_t i;
	
	for(i = 0; i < N_CP_ciMotorInterrupt; i++){
		ciMotorInterrupt_stopMotor(i);
	}
}

/* #[<ENTRY_PORT>]# eiCyclicBody
 * entry port: eiCyclicBody
 * signature:  siHandlerBody
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiCyclicBody_main
 * name:         eiCyclicBody_main
 * global_name:  tRotaryEncoderBody_eiCyclicBody_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eiCyclicBody_main()
{
	if(VAR_initializedFlag){
		VAR_interruptCount = 0;
		sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_IER), MOTOR_INTERRUPT_PINS);
	}
}

/* #[<ENTRY_PORT>]# eiInterruptBody
 * entry port: eiInterruptBody
 * signature:  siHandlerBody
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiInterruptBody_main
 * name:         eiInterruptBody_main
 * global_name:  tRotaryEncoderBody_eiInterruptBody_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
/*
 *  ^C]Ɗ݂N
 */
void nxt_motor_isr_C(void);
void
eiInterruptBody_main()
{
	uint32_t pins;
	int32_t i;

	pins = sil_rew_mem((void *)(ATTR_PIOBase+TOFF_PIO_ISR)); // ݂NA
	pins = sil_rew_mem((void *)(ATTR_PIOBase+TOFF_PIO_PDSR));

	SIL_PRE_LOC;
	SIL_LOC_INT();

	VAR_interruptCount++;
	if(VAR_interruptCount > 4){
		// ݋֎~
		sil_wrw_mem((void *)(ATTR_PIOBase+TOFF_PIO_IDR), MOTOR_INTERRUPT_PINS);
	}

	/* Motor̉]XV */
	for(i = 0; i < N_CP_ciMotorInterrupt; i++){
		ciMotorInterrupt_quadDecode(i, pins);
	}

	SIL_UNL_INT();
}

/* #[<POSTAMBLE>]#
 *   AEA導g[CWSCC
 * #[</POSTAMBLE>]#*/
