/*
 * original:lejos-osek/src/nxtvm/platform/nxt/sensor.c rev.1.1.1.1
 * modified history.
 * 05/25/2010: Modifier Takuya Ishikawa
 *           : Moved init_sensors to tSensorDriverBody.c
 *           : Changed digi0 to SCL, digi1 to SDA
 *           : Changed unset_digi0 to clearSCL
 *           : Changed set_digi0 to setSCL
 *           : Changed unset_digi1 to clearSDA
 *           : Changed set_digi1 to setSDA
 *           : Modified port number to component attribute
 *           : Changed interface name to entry function name with TECS naming rule
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 08/14/2009: Modifider Ryosuke Takeuchi
 *           : Modified Device Interface to ITRON-SIL interface and Changed
 *             AT91SAM7.h to SIL defined include file.
 *           : Modified intenal int function[] for a code size minimum.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tSensorPort_template.c => src/tSensorPort.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]#  #[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * portNumber       uint32_t         ATTR_portNumber 
 *
 * Ăь֐ #_TCPF_#
 * call port : cAVR  signature: sAVR context: task
 *   void           cAVR_update( );
 *   void           cAVR_setMotor( uint32_t portNumber, int32_t powerPercent, int32_t brake );
 *   void           cAVR_powerDown( );
 *   uint32_t       cAVR_getButton( );
 *   uint32_t       cAVR_batteryVoltage( );
 *   uint32_t       cAVR_getSensor( uint32_t portNumber );
 *   void           cAVR_setInputPower( uint32_t portNumber, uint32_t powerType );
 *
 * #[</PREAMBLE>]# */

#include "tSensorPort_tecsgen.h"
#include <sil.h>

static const uint32_t SCL[4] = { (1 << 23), (1 << 28), (1 << 29), (1 << 30) };
static const uint32_t SDA[4] = { (1 << 18), (1 << 19), (1 << 20), (1 << 2) };

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

/* 󂯌֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eSensorPort
 * entry port: eSensorPort
 * signature:  sSensorPort
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eSensorPort_clearSCL
 * name:         eSensorPort_clearSCL
 * global_name:  tSensorPort_eSensorPort_clearSCL
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eSensorPort_clearSCL(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PER), SCL[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_OER), SCL[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_CODR), SCL[ATTR_portNumber]);
}

/* #[<ENTRY_FUNC>]# eSensorPort_clearSDA
 * name:         eSensorPort_clearSDA
 * global_name:  tSensorPort_eSensorPort_clearSDA
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eSensorPort_clearSDA(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PER), SDA[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_OER), SDA[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_CODR), SDA[ATTR_portNumber]);
}

/* #[<ENTRY_FUNC>]# eSensorPort_setSCL
 * name:         eSensorPort_setSCL
 * global_name:  tSensorPort_eSensorPort_setSCL
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eSensorPort_setSCL(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PER), SCL[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_OER), SCL[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_SODR), SCL[ATTR_portNumber]);
}

/* #[<ENTRY_FUNC>]# eSensorPort_setSDA
 * name:         eSensorPort_setSDA
 * global_name:  tSensorPort_eSensorPort_setSDA
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eSensorPort_setSDA(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PER), SDA[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_OER), SDA[ATTR_portNumber]);
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_SODR), SDA[ATTR_portNumber]);
}

/* #[<ENTRY_FUNC>]# eSensorPort_inputPower
 * name:         eSensorPort_inputPower
 * global_name:  tSensorPort_eSensorPort_inputPower
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eSensorPort_inputPower(CELLIDX idx, uint32_t powerType)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	cAVR_setInputPower(ATTR_portNumber, powerType);
}

/* #[<ENTRY_FUNC>]# eSensorPort_getValue
 * name:         eSensorPort_getValue
 * global_name:  tSensorPort_eSensorPort_getValue
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
int32_t
eSensorPort_getValue(CELLIDX idx)
{
	CELLCB	*p_cellcb;
	if (VALID_IDX(idx)) {
		p_cellcb = GET_CELLCB(idx);
	}

	return (int32_t)cAVR_getSensor(ATTR_portNumber);
}

/* #[<POSTAMBLE>]#
 *   艺ɔ󂯌֐܂
 * #[</POSTAMBLE>]#*/
