/*
 * original:lejos-osek/src/nxtvm/platform/nxt/twi.c rev.1.1.1.1
 * modified history.
 * 05/24/2010: Modifier Takuya Ishikawa
 *           : Changed interface name to entry function name with TECS naming rule
 *           : Modified some variables to component variables
 *           : Modified some constants to component attributes
 *           : Modified data type to ITRON spec.
 * 11/04/2008: Modifier takashic
 *           : Modified TWM clock 380KHz to 400KHz.
 * 04/16/2010: Modifider Ryosuke Takeuchi
 *           : Modified Device Interface to ITRON-SIL interface and Changed
 *             AT91SAM7.h to SIL defined include file.
 *           : Deleted UNRE chack in twi_isr_C function.
 *           : Modified Interrupt disable in twi_init fuction for ITRON ATT_INI API.
 */
/*
 * ̃t@C tecsgen ɂev[gƂĎ܂
 * ̃t@CҏWĎgp邱ƂӐ}Ă܂
 * tecsgen ̍Ďsɂ㏑Ă܂߁Aʏ
 *   gen/tTWIBody_template.c => src/tTWIBody.c
 * ̂悤ɖO, tH_ύXĂC܂
 */
/* #[<PREAMBLE>]#
 * #[<...>]# AHA#[</...>]# ň͂܂ꂽRg͕ҏWȂł
 * tecsmerge ɂ}[WɎgp܂
 *
 * ANZX}N #_CAAM_#
 * clockDivisor     uint32_t         ATTR_clockDivisor
 * pending          uint32_t         VAR_pending     
 * buffer           uint8_t*         VAR_buffer      
 * state            uint8_t          VAR_state       
 *
 * #[</PREAMBLE>]# */

#include "tTWIBody_tecsgen.h"
#include "arm_gcc/nxt/at91sam7s.h"

#ifndef E_OK
#define	E_OK	0		/* success */
#define	E_ID	(-18)	/* illegal ID */
#endif

enum TWI_STATE{
	TWI_UNINITIALISED = 0,
	TWI_IDLE,
	TWI_TX_BUSY,
	TWI_TX_DONE,
	TWI_RX_BUSY,
	TWI_RX_DONE,
	TWI_FAILED
};

static inline void
twiReset(void)
{
	volatile uint32_t clocks = 9;

	sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);

	sil_wrw_mem((void *)(TADR_PMC_BASE+TOFF_PMC_PCER), ((1 << INTNO_PIOA_PID) | (1 << INTNO_TWI_PID)));

	/* Set up pin as an IO pin for clocking till clean */
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_MDER), ((1 << 3) | (1 << 4)));
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PER),  ((1 << 3) | (1 << 4)));
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_ODR),  (1 << 3));
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_OER),  (1 << 4));

	while(clocks > 0 && !(sil_rew_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PDSR)) & (1 << 3))){
		sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_CODR), (1 << 4));
		sil_dly_nse(1500*1000);
		sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_SODR), (1 << 4));
		sil_dly_nse(1500*1000);
		clocks--;
	}

	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_PDR), ((1 << 3) | (1 << 4)));
	sil_wrw_mem((void *)(TADR_PIO_BASE+TOFF_PIO_ASR), ((1 << 3) | (1 << 4)));

	sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), 0x88); /* Disable & reset */

	// changed by takashic 2008-11-14
	//  *AT91C_TWI_CWGR = 0x020f0f;	/* Set for 380kHz */
	/* Set for 400kHz */
	sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CWGR), ((ATTR_clockDivisor << 8)|ATTR_clockDivisor));

	/* Enable as master */
	sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), 0x04);
}

/* :֐ #_TEPF_# */
/* #[<ENTRY_PORT>]# eTWI
 * entry port: eTWI
 * signature:  sTWI
 * context:    task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eTWI_initialize
 * name:         eTWI_initialize
 * global_name:  tTWIBody_eTWI_initialize
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
bool_t
eTWI_initialize()
{
	/* Disable all interrupt sources */
	sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);

	twiReset();

	/* Init peripheral */
	VAR_state = TWI_IDLE;

	return true;
}

/* #[<ENTRY_FUNC>]# eTWI_startWrite
 * name:         eTWI_startWrite
 * global_name:  tTWIBody_eTWI_startWrite
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eTWI_startWrite(uint32_t dev_addr, uint32_t int_addr_bytes, uint32_t int_addr, const uint8_t* data, uint32_t nBytes)
{
	volatile uint32_t mode = ((dev_addr & 0x7f) << 16) | ((int_addr_bytes & 3) << 8);

	if(!tTWIBody_eTWI_isBusy()){
		VAR_state = TWI_TX_BUSY;
		/* Disable all interrupts */
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);
		VAR_buffer = data;
		VAR_pending = nBytes;

		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_MMR), mode);
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), (AT91S_TWI_START | AT91S_TWI_MSEN));
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IER), 0x1C4);
	}
}

/* #[<ENTRY_FUNC>]# eTWI_startRead
 * name:         eTWI_startRead
 * global_name:  tTWIBody_eTWI_startRead
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eTWI_startRead(uint32_t dev_addr, uint32_t int_addr_bytes, uint32_t int_addr, uint8_t* data, uint32_t nBytes)
{
	volatile uint32_t mode =
	  ((dev_addr & 0x7f) << 16) | ((int_addr_bytes & 3) << 8) | (1 << 12);
	volatile uint32_t dummy;

	if(!tTWIBody_eTWI_isBusy()){
		VAR_state = TWI_RX_BUSY;
		/* Disable all interrupts */
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);
		VAR_buffer = data;
		VAR_pending = nBytes;
		dummy = sil_rew_mem((void *)(TADR_TWI_BASE+TOFF_TWI_SR));
		dummy = sil_rew_mem((void *)(TADR_TWI_BASE+TOFF_TWI_RHR));
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_MMR), mode);
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), (AT91S_TWI_START | AT91S_TWI_MSEN));
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IER), 0x01C2);
	}
}

/* #[<ENTRY_FUNC>]# eTWI_isBusy
 * name:         eTWI_isBusy
 * global_name:  tTWIBody_eTWI_isBusy
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
bool_t
eTWI_isBusy()
{
	return ((VAR_state == TWI_TX_BUSY) || (VAR_state == TWI_RX_BUSY));
}

/* #[<ENTRY_FUNC>]# eTWI_isOk
 * name:         eTWI_isOk
 * global_name:  tTWIBody_eTWI_isOk
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
bool_t
eTWI_isOk()
{
	return ((VAR_state >= TWI_IDLE) && (VAR_state <= TWI_RX_DONE));
}

/* #[<ENTRY_FUNC>]# eTWI_reset
 * name:         eTWI_reset
 * global_name:  tTWIBody_eTWI_reset
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eTWI_reset()
{
	twiReset();
}

/* #[<ENTRY_PORT>]# eiInterruptBody
 * entry port: eiInterruptBody
 * signature:  siHandlerBody
 * context:    non-task
 * #[</ENTRY_PORT>]# */

/* #[<ENTRY_FUNC>]# eiInterruptBody_main
 * name:         eiInterruptBody_main
 * global_name:  tTWIBody_eiInterruptBody_main
 * oneway:       false
 * #[</ENTRY_FUNC>]# */
void
eiInterruptBody_main()
{
	volatile uint32_t status = sil_rew_mem((void *)(TADR_TWI_BASE+TOFF_TWI_SR));

	/*
	 * 	RvZbT̃f[^M
	 */
	if((status & AT91S_TWI_RXRDY) && VAR_state == TWI_RX_BUSY){
		if(VAR_pending){
			*VAR_buffer = sil_rew_mem((void *)(TADR_TWI_BASE+TOFF_TWI_RHR));
			VAR_buffer++;
			VAR_pending--;

			/*
			 *  Ō1byte
			 */
			if(VAR_pending == 1){
				/* second last byte -- issue a stop on the next byte */
				sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), AT91S_TWI_STOP);
			}

			/*
			 *  MI
			 */
			if(!VAR_pending){
				VAR_state = TWI_RX_DONE;
			}
		}
	}

	/*
	 * 	RvZbTւ̃f[^M
	 */
	if((status & AT91S_TWI_TXRDY) && VAR_state == TWI_TX_BUSY){
		if(VAR_pending){
			/* Still Stuff to send */
			sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), (AT91S_TWI_MSEN | AT91S_TWI_START));
			if(VAR_pending == 1){
				sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), AT91S_TWI_STOP);
			}
			sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_THR), *VAR_buffer);

			VAR_buffer++;
			VAR_pending--;
		} else {
			/* everything has been sent */
			VAR_state = TWI_TX_DONE;
			sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);
		}
	}

	if(status & AT91C_TWI_OVRE) {
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_CR), AT91S_TWI_STOP);
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);	/* Disable all interrupts */
		VAR_state = TWI_FAILED;
	}
	
	if(status & AT91S_TWI_NACK){
		sil_wrw_mem((void *)(TADR_TWI_BASE+TOFF_TWI_IDR), ~0);
		VAR_state = TWI_UNINITIALISED;
	}
}

/* #[<POSTAMBLE>]#
 *   AEA導g[CWSCC
 * #[</POSTAMBLE>]#*/
